<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

if (!defined('_PS_VERSION_'))
  exit;

require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkAPI.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");

class AdminPolarisLinkArchiveController extends ModuleAdminController
{	
	public function __construct()
	{
		$this->context = Context::getContext();			
		$this->bootstrap = true;
					
		parent::__construct();		
	}
    
    /* tri des fichiers par date inverse */
    static function file_sorter($a, $b)
    {
        if ($a["d"] == $b["d"]) {
            return 0;
        }
        return ($a["d"] > $b["d"]) ? -1 : +1;
    }
	
	
	/* On procède à l'affichage */
	public function renderList()
	{		
		$output = '';
		
        $db = Db::getInstance();
        try
		{  
            $sites = $this->module->Config;
            $current_site_id = Tools::getValue('PPL_VEGA_SITE');
            $current_site_id = $current_site_id ? $current_site_id : 0;
            $current_site = null; 

            foreach ($sites as $site)
            {
                if ($site->LinkID == $current_site_id || $current_site_id == 0)
                {
                    $current_site = $site;
                    break;
                }
            }

            if ($current_site)
            {
                $jobsManager = new PolarisLinkJobManager($current_site);

                $paths = array(
                    "[&nbsp;PLANIFIÉ&nbsp;]" => $jobsManager->getQueuePath(),
                    "(&nbspFAIT&nbsp)" => $jobsManager->getArchivesPath()
                );

                $action = Tools::getValue('act');
                $job = Tools::getValue('job');
                $job_file = null;

                if (!empty($job))
                {
                    // recherche du travail
                    $job = str_replace("/", "", $job);
                    $job = str_replace("\\", "", $job);

                    foreach ($paths as $path)
                    {
                        if (file_exists($path . DIRECTORY_SEPARATOR . $job))
                        {
                            $job_file = $path . DIRECTORY_SEPARATOR . $job;
                            break;
                        } 
                    }

                    if (empty($job_file))
                        throw new Exception(sprintf("le travail %s est introuvable !", $job));
                }

                if ($action == "delete")
                {
                    if (empty($job_file))
                        throw new Exception("le travail à supprimer n'a pas été fourni");

                    unlink($job_file);
                }
                if ($action == "delete-all")
                {
                    foreach ($paths as $state => $path)
                    {
                        foreach (scandir($path, SCANDIR_SORT_ASCENDING) as $filename)
                        {
                            if ($filename != "." && $filename != "..")
                                unlink($path."/".$filename);
                        }
                    }
                }
                else if ($action == "download")
                {
                    if (empty($job_file))
                        throw new Exception("le travail à télécharger n'a pas été fourni");

                    if ($jobsManager->testGZfile($job_file))
					{
						header('Content-Type: application/gzip');
						header('Content-Disposition: attachment; filename="'.$job.'.xml.gz');
					}
					else
					{
						header('Content-Type: application/xml; charset=utf-8');
						header('Content-Disposition: attachment; filename="'.$job.'.xml');
					}														
                    readfile($job_file);
                    die();
                }
                else if ($action == "see")
                {
                    if (empty($job_file))
                        throw new Exception("le travail à consulter n'a pas été fourni");

                    $wrapper = "file";
                    if ($jobsManager->testGZfile($job_file))
                        $wrapper = "compress.zlib";
					                    
                    if ($fp = @fopen($wrapper."://".$job_file, "r"))
                    {                        
                        header('Content-Type: application/xml; charset=utf-8');
                        stream_copy_to_stream($fp, fopen("php://output", "w"));
                        fclose($fp);
                    }
                    else
                        echo sprintf("Impossible d'ouvrir le fichier %s : %s !", $job_file, (error_get_last())["message"]);
                    
                    die();
                }
                else if ($action == "test")
                {
                    header("content-type: text/plain;charset=utf-8");
                    if (empty($job_file))
                        throw new Exception("le travail à consulter n'a pas été fourni");

                    $wrapper = "file";
                    if ($jobsManager->testGZfile($job_file))
                        $wrapper = "compress.zlib";
                        
                    try
                    {
                        $docxml = new XMLReader();
			            if (!$docxml->open($wrapper."://".$job_file, 
											NULL, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG | LIBXML_NONET | LIBXML_PARSEHUGE))
                            throw new Exception(sprintf("impossible d'ouvrir le fichier de travail (%s)", $job_file));
                    
                        $emptyCtrl = 0;
                        $depth = 0;	// Profondeur
                        while ($docxml->read())
                        {
                            /*if (!$docxml->isValid())
                                throw new Exception("le fichier soumis n'est pas un XML valide");*/
                        
                            $emptyCtrl++;
                            switch ($docxml->nodeType)
                            {
                                case XMLReader::ELEMENT:
                                    $depth++;
                                    if ($depth == 1)
                                    {
                                        if ($docxml->name != "import")
                                            throw new Exception(sprintf("racine '%s' inconnue dans le document XML, 'import' attendu", $docxml->name));
                                    }
                                    else if ($depth == 2)
                                    {
                                        echo "-" . $docxml->name . "\n";
                                    }
                                    
                                    if ($docxml->isEmptyElement)
                                        $depth--;
                                    break;
                                case XMLReader::END_ELEMENT:							
                                    $depth--;
                                    break;
                            }
                        }
                        
                        $docxml->close();

                        if ($emptyCtrl == 0)
                            throw new Exception("Aucune balise trouvée ?");
                    }
                    catch (\Exception $ex)
                    {
                        echo $ex->getMessage();
                    }
                    
                    die();
                }
            }
                
            // Demande de l'URL !				
            $helper = new HelperForm();
            
            // Module, token and currentIndex
            $helper->module = $this->module;
            $helper->name_controller = "AdminPolarisLinkArchive";
            $helper->token = Tools::getAdminTokenLite('AdminModules');
                    
            // Language
            $id_lang = $this->context->language->id;
            $helper->default_form_language = $id_lang;
            $helper->allow_employee_form_lang = $id_lang;
                    
            // Title and toolbar
            $helper->title = "Liaison PolarisLink";
            $helper->show_toolbar = false;        // false -> remove toolbar
            $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
            $helper->submit_action = 'submit';
                    
            // Init Fields form array		
            $fields_form[0]['form'] = array(
                'input' => array(
                            array(
                                'type' => 'select',
                                'label' => $this->l("Site PolarisLink"),
                                'name' => 'PPL_VEGA_SITE',
                                'required' => true,
                                'options' => array(
                                                'query' => $sites,
                                                'id' => 'LinkID',
                                                'name' => 'LinkName',
                                                )
                            ),
                    ),
                'buttons' => array(
                    'archive' => array(
                        'title' => "Supprimer toutes les archives",
                        'name' => 'delete-all',
                        'type' => 'button',
                        'class' => 'btn btn-default pull-left mr-2',
                        'icon' => 'process-icon-delete',
                        'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkArchive')."&act=delete-all'"
                    )
                ),
                'submit' => array(
                    'title' => "Voir les archives du site",
                    'class' => 'btn btn-default pull-right'
                )
            );	
                                        
            // Load current value
            $helper->fields_value['PPL_VEGA_SITE'] = $current_site_id;
            $output .= $helper->generateForm($fields_form);	
            
            if ($current_site)
            {
                $output .= '<div class="panel">';
                $output .= "<h3>Archives disponibles</h3>";	
                $output .= '<style>';
                $output .= '.redir_tbl { width: 100%; } ';
                $output .= '.redir_tbl tr:hover { background: #ccc; } ';
                $output .= '</style>';
                
                $jobs = [];
                foreach ($paths as $state => $path)
                {
                    foreach (scandir($path, SCANDIR_SORT_ASCENDING) as $filename)
                    {
                        if (strlen($filename) > 3)
                        {
                            $fullpath = $path.DIRECTORY_SEPARATOR.$filename;

                            $jobs[] = [
                                "n" => $filename,
                                "s" => $state,
                                "fs" => filesize($fullpath) / 1024.0,
                                "d" => filemtime($fullpath),
                            ];
                        }
                    }
                }

                usort($jobs, array("AdminPolarisLinkArchiveController", "file_sorter"));

                if (count($jobs) > 0)
                {
                    $output .= "<table class=\"redir_tbl\"><thead><tr><th>ID</th><th>État</th><th>Date</th><th>Taille</th><th></th></tr></thead><tbody>";

                    $paths = array(
                        "[&nbsp;PLANIFIÉ&nbsp;]" => $jobsManager->getQueuePath(),
                        "(&nbspFAIT&nbsp)" => $jobsManager->getArchivesPath()
                    );
                    
                    $url = $this->context->link->getAdminLink('AdminPolarisLinkArchive');

                    foreach ($jobs as $job)
                    {
                        $job_url = $url.'&PPL_VEGA_SITE='.$current_site->LinkID.'&job='.$job["n"];

                        $output .= "<tr>";
                        $output .= "<td>".$job["n"]."</td>";
                        $output .= "<td>".$job["s"]."</td>";
                        $output .= "<td>".date("d/m/Y H:i:s", $job["d"])."</td>";
                        $output .= "<td>".sprintf("%.02f", $job["fs"])."kb</td>";                        
                        $output .= "<td>";
                        $output .= '[&nbsp;<a href="'.$job_url.'&act=download">Télécharger</a>&nbsp;] ';
                        $output .= '[&nbsp;<a target="_blank" href="'.$job_url.'&act=see">Voir</a>&nbsp;] ';
                        $output .= '[&nbsp;<a href="'.$job_url.'&act=test">Tester</a>&nbsp;] ';
                        $output .= '[&nbsp;<a href="'.$job_url.'&act=delete">Supprimer</a>&nbsp;] ';                        
                        $output .= "</td>";
                        $output .= "</tr>";
                    }

                    $output .= "</tbody></table>";
                }
                else
                    $output .= "<p>Il n'y a aucun travail en queue de traitement ou archivé.</p>";

                $output .= "</div>";
            }
        }
        catch (Exception $ex)
        {
            PrestaShopLogger::addLog("Erreur : ".$ex->getMessage(), 2, 1, $this->module->displayName, 1, true);
            $output .= '<div class="alert alert-danger">'.Tools::displayError($ex->getMessage()).'</div>';
        }        
		
		return $output;
	}
}